@@ -6,13 +6,13 @@ from decimal import Decimal
 from dateutil import relativedelta
 from dateutil import rrule
 from itertools import groupby
-from cached_property import cached_property
 
 from trytond.i18n import gettext
 from trytond.model import Workflow, ModelSQL, ModelView, fields, Unique
 from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval, Bool, If
 from trytond.pool import Pool
+from trytond.tools import cached_property
 from trytond.transaction import Transaction
 from trytond.wizard import (Wizard, StateView, StateTransition, StateReport,
     Button)
@@ -94,6 +94,8 @@ class Asset(Workflow, ModelSQL, ModelView):
             },
         depends=['state'],
         required=True)
+    currency = fields.Function(fields.Many2One('currency.currency',
+        'Currency'), 'on_change_with_currency')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
         'on_change_with_currency_digits')
     quantity = fields.Float('Quantity',
@@ -301,9 +303,14 @@ class Asset(Workflow, ModelSQL, ModelView):
             return Decimal(0)
 
     @fields.depends('company')
-    def on_change_with_currency_digits(self, name=None):
+    def on_change_with_currency(self, name=None):
         if self.company:
-            return self.company.currency.digits
+            return self.company.currency.id
+
+    @fields.depends('currency')
+    def on_change_with_currency_digits(self, name=None):
+        if self.currency:
+            return self.currency.digits
         return 2
 
     @fields.depends('supplier_invoice_line', 'unit')
@@ -718,6 +725,8 @@ class AssetLine(ModelSQL, ModelView):
         'Accumulated Depreciation', readonly=True,
         digits=(16, Eval('currency_digits', 2)), depends=['currency_digits'])
     move = fields.Many2One('account.move', 'Account Move', readonly=True)
+    currency = fields.Function(fields.Many2One('currency.currency',
+        'Currency'), 'on_change_with_currency')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
         'on_change_with_currency_digits')
 
@@ -726,6 +735,11 @@ class AssetLine(ModelSQL, ModelView):
         super(AssetLine, cls).__setup__()
         cls._order.insert(0, ('date', 'ASC'))
 
+    @fields.depends('asset', '_parent_asset.currency')
+    def on_change_with_currency(self, name=None):
+        if self.asset:
+            return self.asset.currency.id
+
     @fields.depends('asset', '_parent_asset.currency_digits')
     def on_change_with_currency_digits(self, name=None):
         if self.asset:
@@ -784,8 +798,8 @@ class UpdateAssetShowDepreciation(ModelView):
     amount = fields.Numeric('Amount', readonly=True)
     date = fields.Date('Date', required=True,
         domain=[
-            ('date', '>=', Eval('latest_move_date')),
-            ('date', '<=', Eval('next_depreciation_date')),
+            ('date', '>', Eval('latest_move_date')),
+            ('date', '<', Eval('next_depreciation_date')),
             ],
         depends=['latest_move_date', 'next_depreciation_date'],
         help=('The date must be between the last update/depreciation date '
@@ -817,18 +831,14 @@ class UpdateAsset(Wizard):
     create_lines = StateTransition()
 
     def default_start(self, fields):
-        Asset = Pool().get('account.asset')
-        asset = Asset(Transaction().context['active_id'])
         return {
-            'value': asset.value,
-            'residual_value': asset.residual_value,
-            'end_date': asset.end_date,
+            'value': self.record.value,
+            'residual_value': self.record.residual_value,
+            'end_date': self.record.end_date,
             }
 
     def transition_update_asset(self):
-        Asset = Pool().get('account.asset')
-        asset = Asset(Transaction().context['active_id'])
-        if self.start.value != asset.value:
+        if self.start.value != self.record.value:
             return 'show_move'
         return 'create_lines'
 
@@ -848,15 +858,15 @@ class UpdateAsset(Wizard):
         return min(next_dates)
 
     def default_show_move(self, fields):
-        Asset = Pool().get('account.asset')
-        asset = Asset(Transaction().context['active_id'])
         return {
-            'amount': self.start.value - asset.value,
+            'amount': self.start.value - self.record.value,
             'date': datetime.date.today(),
-            'depreciation_account': asset.product.account_depreciation_used.id,
-            'counterpart_account': asset.product.account_expense_used.id,
-            'latest_move_date': self.get_latest_move_date(asset),
-            'next_depreciation_date': self.get_next_depreciation_date(asset),
+            'depreciation_account': (
+                self.record.product.account_depreciation_used.id),
+            'counterpart_account': self.record.product.account_expense_used.id,
+            'latest_move_date': self.get_latest_move_date(self.record),
+            'next_depreciation_date': self.get_next_depreciation_date(
+                self.record),
             }
 
     def get_move(self, asset):
@@ -889,32 +899,27 @@ class UpdateAsset(Wizard):
     def transition_create_move(self):
         pool = Pool()
         Move = pool.get('account.move')
-        Asset = pool.get('account.asset')
 
-        asset = Asset(Transaction().context['active_id'])
         latest_move_date = self.show_move.latest_move_date
         next_date = self.show_move.next_depreciation_date
-        if not (latest_move_date <= self.show_move.date <= next_date):
+        if not (latest_move_date < self.show_move.date < next_date):
             raise ValueError('The update move date is invalid')
-        move = self.get_move(asset)
-        move.lines = self.get_move_lines(asset)
+        move = self.get_move(self.record)
+        move.lines = self.get_move_lines(self.record)
         move.save()
-        Asset.write([asset], {
+        self.model.write([self.record], {
                 'update_moves': [('add', [move.id])],
                 })
         Move.post([move])
         return 'create_lines'
 
     def transition_create_lines(self):
-        Asset = Pool().get('account.asset')
-
-        asset = Asset(Transaction().context['active_id'])
-        Asset.write([asset], {
+        self.model.write([self.record], {
                 'value': self.start.value,
                 'residual_value': self.start.residual_value,
                 'end_date': self.start.end_date,
                 })
-        Asset.create_lines([asset])
+        self.model.create_lines([self.record])
         return 'end'
 
 
@@ -970,10 +975,7 @@ class AssetDepreciationTable(CompanyReport):
             'closing_value',
             }
         for attr_name in grouped_attributes:
-            descr = cached_property(adder(attr_name))
-            setattr(Grouper, attr_name, descr)
-            if hasattr(descr, '__set_name__'):
-                descr.__set_name__(Grouper, attr_name)
+            setattr(Grouper, attr_name, cached_property(adder(attr_name)))
 
         return Grouper
 
@@ -1113,7 +1115,7 @@ class PrintDepreciationTable(Wizard):
         assets = Asset.search([
                 ('start_date', '<', self.start.end_date),
                 ('end_date', '>', self.start.start_date),
-                ('state', '!=', 'draft'),
+                ('state', '=', 'running'),
                 ])
         return action, {
             'ids': [a.id for a in assets],
